/*
 * This is free and unencumbered software released into the public
 * domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * The complete license can be found in the UNLICENSE file.
 *
 * Vim:
 * set foldmethod=marker
 * set shiftwidth=2
 * set smartcase
 * set tabstop=2
 */

var Toolbox = {};

// Basic format function
Toolbox.Format = function(format) {
  var args = Array.prototype.slice.call(arguments, 1);
  return format.replace(/{(\d+)}/g, function(match, number) { 
    return typeof args[number] != 'undefined'
    ? args[number] 
    : match
    ;
  });
};

Toolbox.Contains = function(inarray, token) {
  var idx;
  var len = inarray.length;
  for(idx = 0; idx < len; idx++) {
    if((inarray[idx]).trim() == token) return true;
  }
  return false;
};

Toolbox.GUID = function() {
    var S4 = function() {
       return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
    };
    return (S4()+S4()+"-"+S4()+"-"+S4()+"-"+S4()+"-"+S4()+S4()+S4());
}

Toolbox.Clone = function(obj) {
    if (null == obj || "object" != typeof obj) return obj;
    var copy = obj.constructor();
    for (var attr in obj) {
        if (obj.hasOwnProperty(attr)) copy[attr] = obj[attr];
    }
    return copy;
}

Toolbox.Dialog = function(pars) {
   this.def = {
     "height": "100px"
    ,"top": "0"
    ,"width": "400px"
    ,"title": new String()
    ,"content": new String()
    ,"align": "center"
    ,"buttons": {}
   }
   $.extend(true,this.def, pars);

   this.dialog = "<div id='toolbox-dialog' style='display:none;'>"
               + "  <div id='toolbox-dialog-title'></div>"
               + "  <div id='toolbox-dialog-content'></div>"
               + "  <div id='toolbox-dialog-footer'></div>"
               + "</div>";

   if($("#toolbox-dialog").length == 0) {
     $("body").append($(this.dialog));
   }
   $("#toolbox-dialog-content").html(this.def.content);
   $("#toolbox-dialog").css({
      "border-style": "solid solid solid solid"
     ,"border-color": "darkgrey"
     ,"border-width": "1px"
     ,"background-color": "#ffffff"
     ,"position": "absolute"
     ,"top": this.def.top
     ,"width": this.def.width
     ,"z-index": "99"
   });
   $("#toolbox-dialog-title").css({
      "border-style": "none none solid none"
     ,"border-color": "darkgrey"
     ,"border-width": "1px"
     ,"background-color": "#eeeeee"
     ,"display": "none"
     ,"font-size": "x-large"
     ,"font-weight": "bold"
     ,"padding": "12px"
     ,"width": "100%"
   });
   $("#toolbox-dialog-content").css({
      "font-size": "large"
     ,"padding": "22px 8px"
     ,"text-align": this.def.align
     ,"width": "100%"
   });
   $("#toolbox-dialog-footer").css({
      "border-style": "none none none none"
     ,"border-color": "darkgrey"
     ,"border-width": "1px"
     ,"background-color": "#ffffff"
     ,"display": "none"
     ,"padding": "5px"
     ,"text-align": "center"
     ,"width": "100%"
   });

   if(this.def.title.length > 0) {
     $("#toolbox-dialog-title")
       .html(this.def.title)
       .show()
       ;
   }

   $("#toolbox-dialog-footer").empty();
   var labels = Object.keys(this.def.buttons);
   if(labels.length > 0) {
     for(var i = 0; i < labels.length; i++) {
       var label = labels[i];
       var button = $("<button type='button'>" + label + "</button>");
       button
         .addClass("btn btn-default btn-block")
         .bind("click", this.def.buttons[label])
         ;
       $("#toolbox-dialog-footer").append(button);
     }
     $("#toolbox-dialog-footer").show();
   }

   this.Show = function(fn) {
     var windowWidth = $(window).width();
     var dialogWidth = $("#toolbox-dialog").width();
     var left = (windowWidth - dialogWidth) / 2;
     $("#toolbox-dialog").css({"left": (left-1) + "px"}); 
     $("#toolbox-dialog").fadeIn("fast", fn); 
   };
   this.Hide = function(fn) {
     $("#toolbox-dialog").fadeOut("fast", fn); 
   };

}

//Toolbox.Dialog = {
//   "pars": {
//     "height": "100px"
//    ,"top": "0"
//    ,"width": "400px"
//    ,"title": new String()
//    ,"content": new String()
//    ,"align": "center"
//    ,"buttons": {}
//   }
//  ,"dialog":   "<div id='toolbox-dialog' style='display:none;'>"
//             + "  <div id='toolbox-dialog-title'>Title</div>"
//             + "  <div id='toolbox-dialog-content'>Hello...</div>"
//             + "  <div id='toolbox-dialog-footer'>Footer</div>"
//             + "</div>"
//  ,"Init": function(pars) {
//    $.extend(true, this.pars, pars);
//
//    if($("#toolbox-dialog").length == 0) {
//      $("body").append($(this.dialog));
//    }
//    $("#toolbox-dialog-title").html(this.pars.title);
//    $("#toolbox-dialog-content").html(this.pars.content);
//    $("#toolbox-dialog").css({
//       "border-style": "solid solid solid solid"
//      ,"border-color": "darkgrey"
//      ,"border-width": "1px"
//      ,"background-color": "#ffffff"
//      ,"position": "absolute"
//      ,"min-height": this.pars.height
//      ,"top": this.pars.top
//      ,"width": this.pars.width
//      ,"z-index": "99"
//    });
//    $("#toolbox-dialog-title").css({
//       "border-style": "none none solid none"
//      ,"border-color": "darkgrey"
//      ,"border-width": "1px"
//      ,"background-color": "#eeeeee"
//      ,"font-size": "x-large"
//      ,"font-weight": "bold"
//      ,"padding": "12px"
//      ,"width": "100%"
//    });
//    $("#toolbox-dialog-content").css({
//       "font-size": "large"
//      ,"padding": "22px 8px"
//      ,"text-align": this.pars.align
//      ,"width": "100%"
//    });
//    $("#toolbox-dialog-footer").css({
//       "border-style": "none none none none"
//      ,"border-color": "darkgrey"
//      ,"border-width": "1px"
//      ,"background-color": "#ffffff"
//      ,"display": "none"
//      ,"padding": "5px"
//      ,"text-align": "center"
//      ,"width": "100%"
//    });
//
//    $("#toolbox-dialog-footer").empty();
//    var labels = Object.keys(this.pars.buttons);
//    if(labels.length > 0) {
//      for(var i = 0; i < labels.length; i++) {
//        var label = labels[i];
//        var button = $("<button type='button'>" + label + "</button>");
//        button
//          .addClass("btn btn-default btn-block")
//          ;
//        $("#toolbox-dialog-footer").append(button);
//      }
//      $("#toolbox-dialog-footer").show();
//    }
//
//  }
//  ,"Show": function(fn) {
//    var windowWidth = $(window).width();
//    var dialogWidth = $("#toolbox-dialog").width();
//    var left = (windowWidth - dialogWidth) / 2;
//    $("#toolbox-dialog").css({"left": (left-1) + "px"}); 
//    $("#toolbox-dialog").fadeIn("fast", fn); 
//  }
//  ,"Hide": function(fn) {
//    $("#toolbox-dialog").fadeOut("fast", fn); 
//  }
//}

Toolbox.Feedback = {
   "Target": new String()
  ,"Message": function(msg, fn) {
    $(this.Target).html(msg).show("slow");
    if(fn) fn($(this.Target));
  }
};

// Simple wrapper for AJAX calls
Toolbox.Ajax = function(url, method, data, success, error) {
  $.ajax({
     url: url
    ,method: method
    ,data: data
    ,success: (success) ? success : function(e) {}
    ,error: (error) ? error : function(e) {}
  });
}
Toolbox.Create = function(url, data, success, error) {
  this.Ajax(url, "POST", data, success, error);
}
Toolbox.Read = function(url, data, success, error) {
  this.Ajax(url, "GET", data, success, error);
}
Toolbox.Update = function(url, data, success, error) {
  this.Ajax(url, "PUT", data, success, error);
}
Toolbox.Delete = function(url, data, success, error) {
  this.Ajax(url, "DELETE", data, success, error);
}

